(* 
 * JABS Demo
 * Wes Weimer
 * Sun May 16 21:22:43 PST 2004
 *
 * Demonstrates:
 *
 * (1) Parsing
 * (2) Visitor Patterns
 * (3) "Internal" Pretty Printing
 * (4) "To Java" Pretty Printing
 *)
open Jabs       (* AST types *)
open Jabsutil   (* Locations *) 
open Jabs_print (* Internal Pretty Printing, Visitors *) 
open Pretty

class visitorOneClass : jabsVisitor = object
  inherit nopJabsVisitor 

  method v_expn expn = 
    let doc = dprintf "  %a -->@!    @[%a@]@!" 
      d_span expn.span d_exp expn.node in
    Printf.printf "%s" (sprint 80 doc) ;
    DoChildren
end 

let visitorOne = new visitorOneClass 

class visitorTwoClass : jabsVisitor = object
  inherit nopJabsVisitor

  method v_qualified_ident qi = ChangeTo(List.map String.uppercase qi)
  method v_ident i = ChangeTo(String.uppercase i) 

  method v_literal l = match l with
    | IntLit(s) -> 
        let i = int_of_string s in
        ChangeTo(IntLit(Printf.sprintf "%d" (i * 2)))
    | _ -> DoChildren
end 

let visitorTwo = new visitorTwoClass 

let main () = 
  for i = 1 to (Array.length Sys.argv) - 1 do
    let filename = Sys.argv.(i) in 
    Printf.printf "Parsing %s:\n\n" filename ;
    let (java_compilation_unit : compilation_unit) =  
      Jabsparse.parse_file filename 
    in 
    let doc = dprintf "  @[%a@]@!" Jabs2java.d_cu java_compilation_unit in 
    Printf.printf "%s" (sprint 80 doc) ;

    Printf.printf "Printing Internal Representation of Expressions:\n\n" ; 
    ignore (visit_compilation_unit visitorOne java_compilation_unit) ; 

    Printf.printf "\nCapitalizing (Qualified.)Identifiers, Doubling Ints:\n";
    let new_cu = visit_compilation_unit visitorTwo java_compilation_unit in
    let doc = dprintf "  @[%a@]@!" Jabs2java.d_cu new_cu in 
    Printf.printf "%s" (sprint 80 doc) ;

    () 
  done 

;;

main () ;;
